"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
var util_1 = require("./jira/util");
var errors_1 = require("./errors");
var date_fns_1 = require("date-fns");
var Parser = /** @class */ (function () {
    function Parser() {
    }
    Parser.parse = function (text) {
        var _this = this;
        var days = text
            .split("#")
            .map(function (d) { return d.trim(); })
            .filter(function (v) { return !!v; });
        return days.map(function (day) {
            var first = day.split("\n")[0].trim();
            var date = util_1.parseDate(first, ["d.M.", "d.M.yy"]);
            if (!date) {
                throw new errors_1.FormatError("Could not parse date '" + first + "'");
            }
            return {
                date: date,
                workEntries: _this.parseDay(date, day)
            };
        });
    };
    Parser.parseDay = function (date, text) {
        var entries = [];
        var lines = text
            .split("\n")
            .slice(1)
            .map(function (l) { return l.trim(); })
            // Filter comments and empty lines
            .filter(function (l) { return l.length > 1 && l[0] !== "/" && l[1] !== "/"; })
            .filter(function (l) { return !!l; });
        var startTime = null;
        var endTime = null;
        var ticketId = null;
        var description = null;
        for (var i = 0; i < lines.length; i += 1) {
            if (!startTime) {
                startTime = this.parseTime(lines[i]);
                continue;
            }
            if (!description || !ticketId) {
                var parts = lines[i].split(":");
                // Add existing description if just adding something to a worklog
                if (parts.length === 1) {
                    ticketId = parts[0].replace("+", "").trim();
                    var existingEntry = __spreadArrays(entries).reverse()
                        .find(function (e) { return e.ticketId === ticketId; });
                    if (!existingEntry) {
                        throw new errors_1.FormatError("Missing Description for Issue '" + ticketId + "' on " + date_fns_1.format(date, "d.M."));
                    }
                    else {
                        description = existingEntry.description;
                    }
                }
                else {
                    ticketId = parts[0].replace("+", "").trim();
                    description = parts
                        .slice(1)
                        .join(":")
                        .trim();
                }
                continue;
            }
            endTime = this.parseTime(lines[i]);
            entries.push({
                startTime: startTime,
                endTime: endTime,
                ticketId: ticketId,
                description: description
            });
            // end time is start time of next
            startTime = endTime;
            // Take breaks into account
            if (i < lines.length - 1 && Parser.lineIsTime(lines[i + 1])) {
                startTime = null;
            }
            endTime = ticketId = description = null;
        }
        return entries;
    };
    Parser.parseTime = function (text) {
        var trimmed = text.replace("+", "").trim();
        var time = util_1.parseDate(trimmed, ["H", "H:m"]);
        if (!time) {
            throw new errors_1.FormatError("Time '" + trimmed + "' is not in right format");
        }
        return time;
    };
    Parser.lineIsTime = function (line) {
        try {
            Parser.parseTime(line);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    return Parser;
}());
exports.Parser = Parser;
